;;; DO NOT MODIFY THIS FILE
(if (featurep 'eicq-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "eicq/_pkg.el")

(package-provide 'eicq :version 1.01 :type 'regular)

;;;***

;;;### (autoloads (eicq-show-window eicq-login eicq-email-author) "eicq" "eicq/eicq.el")

(autoload 'eicq-email-author "eicq" "\
Email comments or money to author." t nil)

(autoload 'eicq-login "eicq" "\
Login to ICQ server.
Make connection to server and network if necessary." t nil)

(autoload 'eicq-show-window "eicq" "\
Show windows of eicq buffers.
Make them if not yet done.
See `eicq-buddy-buffer' and `eicq-log-buffer'." t nil)

;;;***

(provide 'eicq-autoloads)
