;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'eicq-meta '("eicq"))
(custom-add-loads 'eicq '("eicq"))
(custom-add-loads 'eicq-interface '("eicq"))
(custom-add-loads 'eicq-info '("eicq"))
(custom-add-loads 'eicq-sound '("eicq"))
(custom-add-loads 'eicq-option '("eicq-toolbar" "eicq"))
(custom-add-loads 'comm '("eicq"))
(custom-add-loads 'eicq-log '("eicq"))
(custom-add-loads 'eicq-buddy '("eicq"))

;;; custom-load.el ends here
